// Created on: 1991-01-14
// Created by: Arnaud BOUZY
// Copyright (c) 1991-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _Expr_RelationIterator_HeaderFile
#define _Expr_RelationIterator_HeaderFile

#include <Standard.hxx>
#include <Standard_DefineAlloc.hxx>
#include <Standard_Handle.hxx>

#include <Expr_Array1OfSingleRelation.hxx>
#include <Standard_Integer.hxx>
#include <Standard_Boolean.hxx>
class Expr_GeneralRelation;
class Expr_SingleRelation;


//! Iterates on every basic relation contained in
//! a GeneralRelation.
class Expr_RelationIterator 
{
public:

  DEFINE_STANDARD_ALLOC

  
  Standard_EXPORT Expr_RelationIterator(const Handle(Expr_GeneralRelation)& rel);
  
  //! Returns False if no other relation remains.
  Standard_EXPORT Standard_Boolean More() const;
  
  Standard_EXPORT void Next();
  
  //! Returns current basic relation.
  //! Exception is raised if no more relation remains.
  Standard_EXPORT Handle(Expr_SingleRelation) Value() const;




protected:





private:



  Expr_Array1OfSingleRelation myRelation;
  Standard_Integer current;


};







#endif // _Expr_RelationIterator_HeaderFile
