// Created on: 2021-10-25
// Created by: Irina Chesnokova
// Copyright (c) 2021 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.


#ifndef _XCAFDimTolObjects_AngularQualifier_HeaderFile
#define _XCAFDimTolObjects_AngularQualifier_HeaderFile

//! Defines types of qualifier for angular dimensions
enum XCAFDimTolObjects_AngularQualifier
{
XCAFDimTolObjects_AngularQualifier_None,
XCAFDimTolObjects_AngularQualifier_Small,
XCAFDimTolObjects_AngularQualifier_Large,
XCAFDimTolObjects_AngularQualifier_Equal
};

#endif // _XCAFDimTolObjects_AngularQualifier_HeaderFile
