// Created on: 1991-03-27
// Created by: Michel CHAUVAT
// Copyright (c) 1991-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _LProp_CIType_HeaderFile
#define _LProp_CIType_HeaderFile


//! Identifies the type of a particular point on a curve:
//! - LProp_Inflection: a point of inflection
//! - LProp_MinCur: a minimum of curvature
//! - LProp_MaxCur: a maximum of curvature.
enum LProp_CIType
{
LProp_Inflection,
LProp_MinCur,
LProp_MaxCur
};

#endif // _LProp_CIType_HeaderFile
