// Created on: 1991-04-24
// Created by: Arnaud BOUZY
// Copyright (c) 1991-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _Draw_ColorKind_HeaderFile
#define _Draw_ColorKind_HeaderFile


enum Draw_ColorKind
{
Draw_blanc,
Draw_rouge,
Draw_vert,
Draw_bleu,
Draw_cyan,
Draw_or,
Draw_magenta,
Draw_marron,
Draw_orange,
Draw_rose,
Draw_saumon,
Draw_violet,
Draw_jaune,
Draw_kaki,
Draw_corail
};

#endif // _Draw_ColorKind_HeaderFile
