// Created on: 2015-08-10
// Created by: Irina KRYLOVA
// Copyright (c) 2015 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _StepDimTol_GeometricToleranceType_HeaderFile
#define _StepDimTol_GeometricToleranceType_HeaderFile

enum StepDimTol_GeometricToleranceType {
  StepDimTol_GTTAngularityTolerance,
  StepDimTol_GTTCircularRunoutTolerance,
  StepDimTol_GTTCoaxialityTolerance,
  StepDimTol_GTTConcentricityTolerance,
  StepDimTol_GTTCylindricityTolerance,
  StepDimTol_GTTFlatnessTolerance,
  StepDimTol_GTTLineProfileTolerance,
  StepDimTol_GTTParallelismTolerance,
  StepDimTol_GTTPerpendicularityTolerance,
  StepDimTol_GTTPositionTolerance,
  StepDimTol_GTTRoundnessTolerance,
  StepDimTol_GTTStraightnessTolerance,
  StepDimTol_GTTSurfaceProfileTolerance,
  StepDimTol_GTTSymmetryTolerance,
  StepDimTol_GTTTotalRunoutTolerance
};

#endif // _StepDimTol_GeometricToleranceType_HeaderFile