// Created on: 1995-01-25
// Created by: Jean-Louis Frenkel
// Copyright (c) 1995-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _PrsMgr_PresentationManager3d_HeaderFile
#define _PrsMgr_PresentationManager3d_HeaderFile

#include <PrsMgr_PresentationManager.hxx>

Standard_DEPRECATED("Deprecated alias to PrsMgr_PresentationManager")
typedef PrsMgr_PresentationManager PrsMgr_PresentationManager3d;
Standard_DEPRECATED("Deprecated alias to PrsMgr_PresentationManager")
typedef Handle_PrsMgr_PresentationManager Handle_PrsMgr_PresentationManager3d;

#endif // _PrsMgr_PresentationManager3d_HeaderFile
