// Created on: 2012-03-23
// Created by: DBV
// Copyright (c) 2002-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _QABugs_HeaderFile
#define _QABugs_HeaderFile

#include <Standard.hxx>
#include <Standard_DefineAlloc.hxx>
#include <Standard_Handle.hxx>

#include <Draw_Interpretor.hxx>



class QABugs 
{
public:

  DEFINE_STANDARD_ALLOC

  
  Standard_EXPORT static void Commands (Draw_Interpretor& DI);
  
  Standard_EXPORT static void Commands_1 (Draw_Interpretor& DI);
  
  Standard_EXPORT static void Commands_2 (Draw_Interpretor& DI);
  
  Standard_EXPORT static void Commands_3 (Draw_Interpretor& DI);

  Standard_EXPORT static void Commands_5 (Draw_Interpretor& DI);
  
  Standard_EXPORT static void Commands_6 (Draw_Interpretor& DI);
  
  Standard_EXPORT static void Commands_7 (Draw_Interpretor& DI);
  
  Standard_EXPORT static void Commands_8 (Draw_Interpretor& DI);
  
  Standard_EXPORT static void Commands_9 (Draw_Interpretor& DI);
  
  Standard_EXPORT static void Commands_10 (Draw_Interpretor& DI);
  
  Standard_EXPORT static void Commands_11 (Draw_Interpretor& DI);
  
  Standard_EXPORT static void Commands_12 (Draw_Interpretor& DI);
  
  Standard_EXPORT static void Commands_13 (Draw_Interpretor& DI);
  
  Standard_EXPORT static void Commands_14 (Draw_Interpretor& DI);
  
  Standard_EXPORT static void Commands_15 (Draw_Interpretor& DI);
  
  Standard_EXPORT static void Commands_16 (Draw_Interpretor& DI);
  
  Standard_EXPORT static void Commands_17 (Draw_Interpretor& DI);
  
  Standard_EXPORT static void Commands_18 (Draw_Interpretor& DI);
  
  Standard_EXPORT static void Commands_19 (Draw_Interpretor& DI);

  Standard_EXPORT static void Commands_20 (Draw_Interpretor& DI);

  Standard_EXPORT static void Commands_BVH (Draw_Interpretor& DI);

};

#endif // _QABugs_HeaderFile
