// Created on: 1992-11-18
// Created by: Christian CAILLET
// Copyright (c) 1992-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _IFSelect_SelectEntityNumber_HeaderFile
#define _IFSelect_SelectEntityNumber_HeaderFile

#include <Standard.hxx>
#include <Standard_Type.hxx>

#include <IFSelect_SelectBase.hxx>
class IFSelect_IntParam;
class Interface_EntityIterator;
class Interface_Graph;
class TCollection_AsciiString;

class IFSelect_SelectEntityNumber;
DEFINE_STANDARD_HANDLE(IFSelect_SelectEntityNumber, IFSelect_SelectBase)

//! A SelectEntityNumber gets in an InterfaceModel (through a
//! Graph), the Entity which has a specified Number (its rank of
//! adding into the Model) : there can be zero (if none) or one.
//! The Number is not directly defined as an Integer, but as a
//! Parameter, which can be externally controlled
class IFSelect_SelectEntityNumber : public IFSelect_SelectBase
{

public:

  //! Creates a SelectEntityNumber, initially with no specified Number
  Standard_EXPORT IFSelect_SelectEntityNumber();
  
  //! Sets Entity Number to be taken (initially, none is set : 0)
  Standard_EXPORT void SetNumber (const Handle(IFSelect_IntParam)& num);
  
  //! Returns specified Number (as a Parameter)
  Standard_EXPORT Handle(IFSelect_IntParam) Number() const;
  
  //! Returns the list of selected entities : the Entity having the
  //! specified Number (this result assures naturally uniqueness)
  Standard_EXPORT Interface_EntityIterator RootResult (const Interface_Graph& G) const Standard_OVERRIDE;
  
  //! Returns a text defining the criterium : "Entity Number ..."
  Standard_EXPORT TCollection_AsciiString Label() const Standard_OVERRIDE;

  DEFINE_STANDARD_RTTIEXT(IFSelect_SelectEntityNumber,IFSelect_SelectBase)

private:

  Handle(IFSelect_IntParam) thenum;

};

#endif // _IFSelect_SelectEntityNumber_HeaderFile
