// Created on: 1995-12-01
// Created by: EXPRESS->CDL V0.2 Translator
// Copyright (c) 1995-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _StepBasic_SiUnitName_HeaderFile
#define _StepBasic_SiUnitName_HeaderFile


enum StepBasic_SiUnitName
{
StepBasic_sunMetre,
StepBasic_sunGram,
StepBasic_sunSecond,
StepBasic_sunAmpere,
StepBasic_sunKelvin,
StepBasic_sunMole,
StepBasic_sunCandela,
StepBasic_sunRadian,
StepBasic_sunSteradian,
StepBasic_sunHertz,
StepBasic_sunNewton,
StepBasic_sunPascal,
StepBasic_sunJoule,
StepBasic_sunWatt,
StepBasic_sunCoulomb,
StepBasic_sunVolt,
StepBasic_sunFarad,
StepBasic_sunOhm,
StepBasic_sunSiemens,
StepBasic_sunWeber,
StepBasic_sunTesla,
StepBasic_sunHenry,
StepBasic_sunDegreeCelsius,
StepBasic_sunLumen,
StepBasic_sunLux,
StepBasic_sunBecquerel,
StepBasic_sunGray,
StepBasic_sunSievert
};

#endif // _StepBasic_SiUnitName_HeaderFile
