// Created on: 1995-12-01
// Created by: EXPRESS->CDL V0.2 Translator
// Copyright (c) 1995-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _StepBasic_SiPrefix_HeaderFile
#define _StepBasic_SiPrefix_HeaderFile


enum StepBasic_SiPrefix
{
StepBasic_spExa,
StepBasic_spPeta,
StepBasic_spTera,
StepBasic_spGiga,
StepBasic_spMega,
StepBasic_spKilo,
StepBasic_spHecto,
StepBasic_spDeca,
StepBasic_spDeci,
StepBasic_spCenti,
StepBasic_spMilli,
StepBasic_spMicro,
StepBasic_spNano,
StepBasic_spPico,
StepBasic_spFemto,
StepBasic_spAtto
};

#endif // _StepBasic_SiPrefix_HeaderFile
