// Created on: 2008-05-29
// Created by: Sergey ZARITCHNY
// Copyright (c) 2008-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _TDataXtd_ConstraintEnum_HeaderFile
#define _TDataXtd_ConstraintEnum_HeaderFile

//! The terms of this enumeration define the types
//! of  available constraint.
//! ==================
enum TDataXtd_ConstraintEnum
{
TDataXtd_RADIUS,
TDataXtd_DIAMETER,
TDataXtd_MINOR_RADIUS,
TDataXtd_MAJOR_RADIUS,
TDataXtd_TANGENT,
TDataXtd_PARALLEL,
TDataXtd_PERPENDICULAR,
TDataXtd_CONCENTRIC,
TDataXtd_COINCIDENT,
TDataXtd_DISTANCE,
TDataXtd_ANGLE,
TDataXtd_EQUAL_RADIUS,
TDataXtd_SYMMETRY,
TDataXtd_MIDPOINT,
TDataXtd_EQUAL_DISTANCE,
TDataXtd_FIX,
TDataXtd_RIGID,
TDataXtd_FROM,
TDataXtd_AXIS,
TDataXtd_MATE,
TDataXtd_ALIGN_FACES,
TDataXtd_ALIGN_AXES,
TDataXtd_AXES_ANGLE,
TDataXtd_FACES_ANGLE,
TDataXtd_ROUND,
TDataXtd_OFFSET
};

#endif // _TDataXtd_ConstraintEnum_HeaderFile
