// Created on: 1996-12-16
// Created by: Bruno DUMORTIER
// Copyright (c) 1996-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _BiTgte_ContactType_HeaderFile
#define _BiTgte_ContactType_HeaderFile


enum BiTgte_ContactType
{
BiTgte_FaceFace,
BiTgte_FaceEdge,
BiTgte_FaceVertex,
BiTgte_EdgeEdge,
BiTgte_EdgeVertex,
BiTgte_VertexVertex
};

#endif // _BiTgte_ContactType_HeaderFile
