// Created on: 1994-09-02
// Created by: Yves FRICAUD
// Copyright (c) 1994-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _LProp_AnalyticCurInf_HeaderFile
#define _LProp_AnalyticCurInf_HeaderFile

#include <Standard.hxx>
#include <Standard_DefineAlloc.hxx>

#include <GeomAbs_CurveType.hxx>
class LProp_CurAndInf;


//! Computes the locals extremas of curvature of a gp curve
//! Remark : a gp curve has not inflection.
class LProp_AnalyticCurInf 
{
public:

  DEFINE_STANDARD_ALLOC

  
  Standard_EXPORT LProp_AnalyticCurInf();
  
  Standard_EXPORT void Perform (const GeomAbs_CurveType T, const Standard_Real UFirst, const Standard_Real ULast, LProp_CurAndInf& Result);




protected:





private:





};







#endif // _LProp_AnalyticCurInf_HeaderFile
