// Created on: 2015-08-06
// Created by: Ilya Novikov
// Copyright (c) 2004-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.


#ifndef _XCAFDimTolObjects_DatumSingleModif_HeaderFile
#define _XCAFDimTolObjects_DatumSingleModif_HeaderFile

//! Defines modifirs
enum XCAFDimTolObjects_DatumSingleModif
{
XCAFDimTolObjects_DatumSingleModif_AnyCrossSection,
XCAFDimTolObjects_DatumSingleModif_Any_LongitudinalSection,
XCAFDimTolObjects_DatumSingleModif_Basic,
XCAFDimTolObjects_DatumSingleModif_ContactingFeature,
XCAFDimTolObjects_DatumSingleModif_DegreeOfFreedomConstraintU,
XCAFDimTolObjects_DatumSingleModif_DegreeOfFreedomConstraintV,
XCAFDimTolObjects_DatumSingleModif_DegreeOfFreedomConstraintW,
XCAFDimTolObjects_DatumSingleModif_DegreeOfFreedomConstraintX,
XCAFDimTolObjects_DatumSingleModif_DegreeOfFreedomConstraintY,
XCAFDimTolObjects_DatumSingleModif_DegreeOfFreedomConstraintZ,
XCAFDimTolObjects_DatumSingleModif_DistanceVariable,
XCAFDimTolObjects_DatumSingleModif_FreeState,
XCAFDimTolObjects_DatumSingleModif_LeastMaterialRequirement,
XCAFDimTolObjects_DatumSingleModif_Line,
XCAFDimTolObjects_DatumSingleModif_MajorDiameter,
XCAFDimTolObjects_DatumSingleModif_MaximumMaterialRequirement,
XCAFDimTolObjects_DatumSingleModif_MinorDiameter,
XCAFDimTolObjects_DatumSingleModif_Orientation,
XCAFDimTolObjects_DatumSingleModif_PitchDiameter,
XCAFDimTolObjects_DatumSingleModif_Plane,
XCAFDimTolObjects_DatumSingleModif_Point,
XCAFDimTolObjects_DatumSingleModif_Translation
};

#endif // _XCAFDimTolObjects_DatumSingleModif_HeaderFile
